/*
    v0.01 - original

*/


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <tap.h>
#include "exTap.h"
#include "tf5800.h"
//#include "libFirebird.h"
//#include "fnames1408.h"


#define ID_USERTAP 0xbdb03801
TAP_ID( ID_USERTAP );

TAP_PROGRAM_NAME("version_detect v0.01");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("a tap to check the firmware version");
TAP_ETCINFO(__DATE__);

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
#include "TSRCommander.inc"

void TSRCommanderConfigDialog();
bool TSRCommanderExitTAP();
int is_tsr_commander_running(void);

//----------------------------------------------------------------------------
//
//main
//
type_menu main_menu;

int TAP_Main(void);
dword TAP_EventHandler(word event, dword param1, dword param2);
int main_menu_handler(type_menu *menu, dword param1);
int key_handler(dword param1);

int init(void);
int close(void);

int debug = 1;
int version_status();
//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
bool TSRCommanderExitTAP()
{
    close();
    TAP_Exit();
    return FALSE;
}

void TSRCommanderConfigDialog()
{
    TAP_ExitNormal();
    ShowMenu(&main_menu);
}

int is_tsr_commander_running(void)
{
    int i;
	type_TAPProcess *TAPProcess;

    if (fw_tap_table <= 0) return 0;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x807a10b2) {  //tsr_commander
                    return 1;
                }
                if (TAPProcess->header->id == 0x81001021) {  //tap_commander
                    return 1;
                }
            }
        }
    }
    return 0;
}

//----------------------------------------------------------------------------
//
//print multiline message to OSD
//  500=vertical position(0-576)
//  RGB(3,5,10)=background colour(dark blue)
//  300=hold time(3 seconds)
//  0=blocking
//
void oprintf(const void *fmt, ...)
{
    va_list argp;

    va_start(argp, fmt);
    vsprintf(strLog, fmt, argp);
    if (debug) TAP_Print("\n%s", strLog);
    ShowMessage_y(strLog, 500, RGB(3,5,10), 300, 0);
    va_end(argp);

    return;
}

void xyprintf(int x, int y, const void *fmt, ...)
{
    va_list argp;

    va_start(argp, fmt);
    vsprintf(strLog, fmt, argp);
    if (debug) TAP_Print("\n%s", strLog);
    ShowMessage_xy(strLog, x, y, COLOR_White, RGB(3,5,10), 100, 0);
    va_end(argp);

    return;
}

//----------------------------------------------------------------------------
//
//main
//
char *tap_name;
type_menu main_menu = {
    0,
    0,
    {0},
    0,
    (char *)__tap_program_name__,
    {
        {2, "xxx",                 },
        {0, "Status",              },
        {1, "Exit Tap",            },
    },
};

//----------------------------------------------------------------------------
int TAP_Main(void)
{
    int ret;

    tap_name =  (char *)__tap_program_name__;
    sprintf(strLog, "Opening %s", tap_name);
    log_message(LOG_SERIAL, strLog, DELAY_SHORT);

    //determine support for different firmwares
    ret = exTAP_Init();
    if (ret < 0) {
        sprintf(strLog, "Unsupported platform - %d:%04x", fw_sysid, _appl_version);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_LONG);
//        return 0;
    }

    init();
    sprintf(strLog, "Opening %s", tap_name);
    log_message(LOG_SCREEN_CONT, strLog, DELAY_SHORT);

    return 1;                 //0=application mode, 1=tsr mode
}


int init(void)
{
    main_menu.visible = 0;

    TSRCommanderInit(0, TRUE);
    return 0;
}

int close(void)
{
    DeleteMenu(&main_menu);

    sprintf(strLog, "Closing  %s", tap_name);
    log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (message_rgn) TAP_Osd_Delete(message_rgn);
    if (OpLog) TAP_Hdd_Fclose(OpLog);
    TSRCommanderErase();
    if (debug) TAP_Print("\nClosing %s", tap_name);
    return 0;
}

//----------------------------------------------------------------------------
dword TAP_EventHandler(word event, dword param1, dword param2)
{
    dword tick;
    dword state, subState;

    switch (event) {
        case EVT_IDLE:
//            TAP_Print("\nEVT_IDLE, event=%x, param1=%x, param2=%x", event, param1, param2);
            tick = TAP_GetTick();

            if (message_showing) {
                if (tick > message_showing) {
                    TAP_EnterNormal();
                    TAP_Osd_Delete(message_rgn);
                    message_showing = 0;
                    message_rgn = 0;
                }
            }

    		TSRCommanderWork();
            break;

        case EVT_KEY:
//            TAP_Print("\nEVT_KEY, event=%x, param1=%x, param2=%x", event, param1, param2);
                TAP_GetState(&state, &subState);
//            TAP_Print("\nEVT_KEY, state=%x, subState=%x", state, subState);

            if (main_menu.visible) {
                param1 = main_menu_handler(&main_menu, param1);
            }
            else
            if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                param1 = key_handler(param1);
            }
            break;

        default:
//            TAP_Print("\nEVT_???, event=%x, param1=%x, param2=%x", event, param1, param2);
            break;
    }
    return param1;
}

int main_menu_handler(type_menu *menu, dword param1)
{
    int selection;

    switch (param1) {
        case RKEY_Ok:
            selection = TAP_Win_GetSelection(&menu->win);
            DeleteMenu(menu);
            TAP_EnterNormal();
            switch (selection) {
                case 0:
                    version_status();
                    return 0;

                case 1:
                    close();
                    TAP_Exit();
                    return 0;
            }
            break;
        case RKEY_Exit:
            DeleteMenu(menu);
            TAP_EnterNormal();
            return 0;
        default:
            if (param1 == RKEY_Recall) {
                 DeleteMenu(menu);
                 TAP_EnterNormal();
                 return param1;
            }
            TAP_Win_Action(&menu->win, param1);
            return 0;
    }
    return param1;
}

int key_handler(dword param1)
{
    if (param1 == RKEY_Recall) {
        //only if tsrcommander is not running
        if (!is_tsr_commander_running()) {
            TAP_ExitNormal();
            ShowMenu(&main_menu);
            return 0;
        }
    }
    return param1;
}

//----------------------------------------------------------------------------
#include "instructions.h"       //exTap

int version_status()
{
	type_TAPProcess *TAPProcess;


    fw_TAP_GetSignalLevel = (void *)FindInstructionPattern(&pattern_fw_TAP_GetSignalLevel);
    sprintf(strLog, "%08x          fw_TAP_GetSignalLevel", fw_TAP_GetSignalLevel);
log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (fw_TAP_GetSignalLevel == 0) goto ret;

    fw_tap_ident     = get_variable(fw_TAP_GetSignalLevel, 0x0a, 0x12);
    sprintf(strLog, "%08x          fw_tap_ident", fw_tap_ident);
log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (fw_tap_ident == 0) goto ret;

    fw_tap_table     = get_variable(fw_TAP_GetSignalLevel, 0x1e, 0x22);
    sprintf(strLog, "%08x          fw_tap_table", fw_tap_table);
log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (fw_tap_table == 0) goto ret;

//    fw_gp            = get_variable((void *)0x80000000, 0x26, 0x2a);
    if (is_firmware(fw_tap_table) && is_firmware(fw_tap_ident)) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (*(dword *)fw_tap_ident*32));
        if (is_firmware(TAPProcess)) {
            fw_gp = TAPProcess->firmwaregp;
        }
    }
    sprintf(strLog, "%08x          fw_gp", fw_gp);
log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (fw_gp == 0) goto ret;

    fw_ioctl         = (void *)FindInstructionPattern(&pattern_fw_ioctl);
    sprintf(strLog, "%08x          fw_ioctl", fw_ioctl);
log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (fw_ioctl == 0) goto ret;

    fw_add_task_queue       = (void *)FindInstructionPattern(&pattern_fw_add_task_queue);
    sprintf(strLog, "%08x          fw_add_task_queue", fw_add_task_queue);
log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (fw_add_task_queue == 0) goto ret;

    fw_etc_structure = get_gpvar(fw_gp, fw_TAP_GetSignalLevel, 0x4a);
    sprintf(strLog, "%08x          fw_etc_structure", fw_etc_structure);
log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);
    if (fw_etc_structure == 0) goto ret;

    sprintf(strLog, "%d:%04x", fw_sysid, _appl_version);
ret:
    log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_LONG);
    return 0;
}

